import { Injectable } from '@/core/api/di/decorators'
import { environment } from '@/core/api/environment'
import { RequestContext } from '@/libs/domain/contexts/request.context'
import { CustomerEntity } from '@/libs/domain/entities/tenant/customer.entity'
import { EncryptHelper } from '@/libs/domain/helpers/encrypt.helper'
import { TokenHelper } from '@/libs/domain/helpers/token.helper'
import { MailMessageBuilder } from '@/libs/domain/notification/mail-message-builder'
import { CustomerRepository } from '@/libs/domain/repositories/tenant/customer.repository'
import { RoleRepository } from '@/libs/domain/repositories/tenant/role.repository'
import { UserRepository } from '@/libs/domain/repositories/tenant/user.repository'
import { MailerService } from '@/libs/domain/services/mailer.service'
import { Role } from '@/libs/infrastructure/persistance/seeders/dev.seeder'

@Injectable()
export class CreateCustomerUseCase {
  constructor(
    private readonly userRepository: UserRepository,
    private readonly roleRepository: RoleRepository,
    private readonly customerRepository: CustomerRepository,
    private readonly mailerService: MailerService
  ) {}

  async run(payload: CustomerEntity, { tenant }: RequestContext): Promise<CustomerEntity> {
    const customerRole = await this.roleRepository.findOne(
      {
        name: Role.customer,
      },
      tenant.id
    )

    const password = payload.user?.password ?? Math.random().toString(36).slice(-8)
    const hashedPassword = await EncryptHelper.hash(password)

    const user = await this.userRepository.save(
      {
        ...payload.user!,
        password: hashedPassword,
        roles: [customerRole!],
      },
      tenant.id
    )
    const customer = await this.customerRepository.save({ ...payload, user }, tenant.id)

    const token = await TokenHelper.generateJWT({ id: user.id })

    const resetPasswordUrl = `${environment.BACKOFFICE_URL}/reset-password/${token}`

    if (!payload.user?.isActive) {
      await this.mailerService.sendEmail(
        new MailMessageBuilder({
          from: `${environment.API_NAME} <${environment.API_EMAIL}>`,
          to: payload.user?.email,
          subject: 'Establecer contraseña',
        })
          .addTitle(`Estimado ${user.firstName} ${user.lastName}`)
          .addLine(
            `Usted está recibiendo este correo electrónico por que ha sido agregado a ${tenant.legalName} como cliente.`
          )
          .addLine(`Para ello se requiere que establezca una contraseña.`)
          .addAction('Establecer contraseña', resetPasswordUrl)
          .addLine(`Si no solicitó unirse, no se requiere ninguna acción adicional.`)
          .build()
      )
    }

    return customer
  }
}
