import { Injectable } from '@/core/api/di/decorators'
import { environment } from '@/core/api/environment'
import { RequestContext } from '@/libs/domain/contexts/request.context'
import { DriverEntity } from '@/libs/domain/entities/tenant/driver.entity'
import { EncryptHelper } from '@/libs/domain/helpers/encrypt.helper'
import { TokenHelper } from '@/libs/domain/helpers/token.helper'
import { MailMessageBuilder } from '@/libs/domain/notification/mail-message-builder'
import { DriverRepository } from '@/libs/domain/repositories/tenant/driver.repository'
import { RoleRepository } from '@/libs/domain/repositories/tenant/role.repository'
import { TransportCompanyRepository } from '@/libs/domain/repositories/tenant/transport-company.repository'
import { UserRepository } from '@/libs/domain/repositories/tenant/user.repository'
import { MailerService } from '@/libs/domain/services/mailer.service'
import { Role } from '@/libs/infrastructure/persistance/seeders/dev.seeder'

@Injectable()
export class CreateDriverUseCase {
  constructor(
    private readonly driverRepository: DriverRepository,
    private readonly userRepository: UserRepository,
    private readonly transportCompanyRepository: TransportCompanyRepository,
    private readonly roleRepository: RoleRepository,
    private readonly mailerService: MailerService
  ) {}

  async run(
    { transportCompanyId, ...payload }: DriverEntity,
    { tenant }: RequestContext
  ): Promise<DriverEntity> {
    const transportCompany = await this.transportCompanyRepository.findOne(
      { id: transportCompanyId },
      tenant.id
    )

    const driverRole = await this.roleRepository.findOne(
      {
        name: Role.driver,
      },
      tenant.id
    )

    const password = payload.user?.password ?? Math.random().toString(36).slice(-8)
    const hashedPassword = await EncryptHelper.hash(password)

    const user = await this.userRepository.save(
      {
        ...payload.user,
        password: hashedPassword,
        roles: [driverRole!],
      },
      tenant.id
    )

    const driver = await this.driverRepository.save(
      {
        ...payload,
        user,
        transportCompany: transportCompany!,
      },
      tenant.id
    )

    const token = await TokenHelper.generateJWT({ id: user.id })

    const resetPasswordUrl = `${environment.BACKOFFICE_URL}/reset-password/${token}`

    if (!payload.user?.isActive) {
      await this.mailerService.sendEmail(
        new MailMessageBuilder({
          from: `${environment.API_NAME} <${environment.API_EMAIL}>`,
          to: payload.user?.email,
          subject: 'Establecer contraseña',
        })
          .addTitle(`Estimado ${user.firstName} ${user.lastName}`)
          .addLine(
            `Usted está recibiendo este correo electrónico por que ha sido agregado a ${transportCompany?.legalName} como conductor.`
          )
          .addLine(`Para ello se requiere que establezca una contraseña.`)
          .addAction('Establecer contraseña', resetPasswordUrl)
          .addLine(`Si no solicitó unirse, no se requiere ninguna acción adicional.`)
          .build()
      )
    }

    return driver
  }
}
